@@ -18,9 +18,15 @@ module GHC.RTS.Events (
        ThreadId,
        TaskId,
        KernelThreadId(..),
+       -- some types for the parallel RTS
+       ProcessId,
+       MachineId,
+       PortId,
+       MessageSize,
+       MessageTag(..),
 
        -- * Reading and writing event logs
-       readEventLogFromFile,
+       readEventLogFromFile, getEventLog,
        writeEventLogToFile,
 
        -- * Utilities
@@ -45,7 +51,7 @@ import Control.Monad
 import Data.IntMap (IntMap)
 import qualified Data.IntMap as M
 import Control.Monad.Reader
-import Control.Monad.Error
+import Control.Monad.Except
 import qualified Data.ByteString.Lazy as L
 import Data.Function
 import Data.List
@@ -135,7 +141,7 @@ standardParsers = [
       lbs <- lift . lift $ getLazyByteString ((fromIntegral block_size) -
                                               (fromIntegral sz_block_event))
       eparsers <- ask
-      let e_events = runGet (runErrorT $ runReaderT (getEventBlock eparsers) eparsers) lbs
+      let e_events = runGet (runExceptT $ runReaderT (getEventBlock eparsers) eparsers) lbs
       return EventBlock{ end_time=end_time,
                          cap= fromIntegral c,
                          block_events=case e_events of
@@ -333,30 +339,6 @@ ghc7Parsers = [
       return RunThread{thread=t}
    )),
 
- (FixedSizeParser EVENT_STOP_THREAD (sz_tid + sz_th_stop_status) (do
-      -- (thread, status)
-      t <- getE
-      s <- getE :: GetEvents RawThreadStopStatus
-      return StopThread{thread=t, status = if s > maxThreadStopStatus
-                                              then NoStatus
-                                              else mkStopStatus s}
-   )),
-
- (FixedSizeParser EVENT_STOP_THREAD (sz_tid + sz_th_stop_status + sz_tid) (do
-      -- (thread, status, info)
-      t <- getE
-      s <- getE :: GetEvents RawThreadStopStatus
-      i <- getE :: GetEvents ThreadId
-      return StopThread{thread = t,
-                        status = case () of
-                                  _ | s > maxThreadStopStatus
-                                    -> NoStatus
-                                    | s == 8 {- XXX yeuch -}
-                                    -> BlockedOnBlackHoleOwnedBy i
-                                    | otherwise
-                                    -> mkStopStatus s}
-   )),
-
  (FixedSizeParser EVENT_THREAD_RUNNABLE sz_tid (do  -- (thread)
       t <- getE
       return ThreadRunnable{thread=t}
@@ -439,6 +421,80 @@ ghc7Parsers = [
    ))
  ]
 
+-- special thread stop event parsers for GHC version 7.8.2
+-- see [Stop status in GHC-7.8.2] in EventTypes.hs
+ghc782StopParser :: EventParser EventInfo
+ghc782StopParser =
+ (FixedSizeParser EVENT_STOP_THREAD (sz_tid + sz_th_stop_status + sz_tid) (do
+      -- (thread, status, info)
+      t <- getE
+      s <- getE :: GetEvents RawThreadStopStatus
+      i <- getE :: GetEvents ThreadId
+      return StopThread{thread = t,
+                        status = case () of
+                                  _ | s > maxThreadStopStatus782
+                                    -> NoStatus
+                                    | s == 9 {- XXX yeuch -}
+                                      -- GHC-7.8.2: 9 == BlockedOnBlackHole
+                                    -> BlockedOnBlackHoleOwnedBy i
+                                    | otherwise
+                                    -> mkStopStatus782 s}
+   ))
+
+-- parsers for GHC < 7.8.2. Older versions do not use block info
+-- (different length).  See [Stop status in GHC-7.8.2] in
+-- EventTypes.hs
+pre77StopParsers :: [EventParser EventInfo]
+pre77StopParsers = [
+ (FixedSizeParser EVENT_STOP_THREAD (sz_tid + sz_th_stop_status) (do
+      -- (thread, status)
+      t <- getE
+      s <- getE :: GetEvents RawThreadStopStatus
+      return StopThread{thread=t, status = if s > maxThreadStopStatusPre77
+                                              then NoStatus
+                                              else mkStopStatus s}
+                        -- older version of the event, no block info
+   )),
+
+ (FixedSizeParser EVENT_STOP_THREAD (sz_tid + sz_th_stop_status + sz_tid) 
+    (do
+      -- (thread, status, info)
+      t <- getE
+      s <- getE :: GetEvents RawThreadStopStatus
+      i <- getE :: GetEvents ThreadId
+      return StopThread{thread = t,
+                        status = case () of
+                                  _ | s > maxThreadStopStatusPre77
+                                    -> NoStatus
+                                    | s == 8 {- XXX yeuch -}
+                                      -- pre-7.7: 8==BlockedOnBlackhole
+                                    -> BlockedOnBlackHoleOwnedBy i
+                                    | otherwise
+                                    -> mkStopStatus s}
+    ))
+  ]
+
+-- parsers for GHC >= 7.8.3, always using block info field parser.
+-- See [Stop status in GHC-7.8.2] in EventTypes.hs
+post782StopParser :: EventParser EventInfo
+post782StopParser = 
+ (FixedSizeParser EVENT_STOP_THREAD (sz_tid + sz_th_stop_status + sz_tid) 
+    (do
+      -- (thread, status, info)
+      t <- getE
+      s <- getE :: GetEvents RawThreadStopStatus
+      i <- getE :: GetEvents ThreadId
+      return StopThread{thread = t,
+                        status = case () of
+                                  _ | s > maxThreadStopStatus
+                                    -> NoStatus
+                                    | s == 8 {- XXX yeuch -}
+                                      -- post-7.8.2: 8==BlockedOnBlackhole
+                                    -> BlockedOnBlackHoleOwnedBy i
+                                    | otherwise
+                                    -> mkStopStatus s}
+    ))
+
  -----------------------
  -- GHC 6.12 compat: GHC 6.12 reported the wrong sizes for some events,
  -- so we have to recognise those wrong sizes here for backwards
@@ -464,11 +520,13 @@ ghc6Parsers = [
 
  (FixedSizeParser EVENT_STOP_THREAD (sz_old_tid + 2) (do  -- (thread, status)
       t <- getE
-      s <- getE :: GetEvents Word16
-      let stat = fromIntegral s
-      return StopThread{thread=t, status = if stat > maxBound
+      s <- getE :: GetEvents RawThreadStopStatus
+      return StopThread{thread=t, status = if s > maxThreadStopStatusPre77
                                               then NoStatus
-                                              else mkStopStatus stat}
+                                              else mkStopStatus s}
+                        -- older version of the event uses pre-77 encoding
+                        -- (actually, it only uses encodings 0 to 5)
+                        -- see [Stop status in GHC-7.8.2] in EventTypes.hs
    )),
 
  (FixedSizeParser EVENT_THREAD_RUNNABLE sz_old_tid (do  -- (thread)
@@ -512,6 +570,103 @@ ghc6Parsers = [
    ))
  ]
 
+-- Parsers for parallel events. Parameter is the thread_id size, to create
+-- ghc6-parsers (using the wrong size) where necessary.
+parRTSParsers :: EventTypeSize -> [EventParser EventInfo]
+parRTSParsers sz_tid = [
+ (VariableSizeParser EVENT_VERSION (do -- (version)
+      num <- getE :: GetEvents Word16
+      string <- getString num
+      return Version{ version = string }
+   )),
+
+ (VariableSizeParser EVENT_PROGRAM_INVOCATION (do -- (cmd. line)
+      num <- getE :: GetEvents Word16
+      string <- getString num
+      return ProgramInvocation{ commandline = string }
+   )),
+
+ (simpleEvent EVENT_EDEN_START_RECEIVE EdenStartReceive),
+ (simpleEvent EVENT_EDEN_END_RECEIVE   EdenEndReceive),
+
+ (FixedSizeParser EVENT_CREATE_PROCESS sz_procid
+    (do p <- getE
+        return CreateProcess{ process = p })
+ ),
+
+ (FixedSizeParser EVENT_KILL_PROCESS sz_procid
+    (do p <- getE
+        return KillProcess{ process = p })
+ ),
+
+ (FixedSizeParser EVENT_ASSIGN_THREAD_TO_PROCESS (sz_tid + sz_procid)
+    (do t <- getE
+        p <- getE
+        return AssignThreadToProcess { thread = t, process = p })
+ ),
+
+ (FixedSizeParser EVENT_CREATE_MACHINE (sz_mid + sz_realtime)
+    (do m <- getE
+        t <- getE
+        return CreateMachine { machine = m, realtime = t })
+ ),
+
+ (FixedSizeParser EVENT_KILL_MACHINE sz_mid
+    (do m <- getE :: GetEvents MachineId
+        return KillMachine { machine = m })
+ ),
+
+ (FixedSizeParser EVENT_SEND_MESSAGE
+    (sz_msgtag + 2*sz_procid + 2*sz_tid + sz_mid)
+    (do tag <- getE :: GetEvents RawMsgTag
+        sP  <- getE :: GetEvents ProcessId
+        sT  <- getE :: GetEvents ThreadId
+        rM  <- getE :: GetEvents MachineId
+        rP  <- getE :: GetEvents ProcessId
+        rIP <- getE :: GetEvents PortId
+        return SendMessage { mesTag = toMsgTag tag,
+                             senderProcess = sP,
+                             senderThread = sT,
+                             receiverMachine = rM,
+                             receiverProcess = rP,
+                             receiverInport = rIP
+                           })
+ ),
+
+ (FixedSizeParser EVENT_RECEIVE_MESSAGE
+    (sz_msgtag + 2*sz_procid + 2*sz_tid + sz_mid + sz_mes)
+    (do tag <- getE :: GetEvents Word8
+        rP  <- getE :: GetEvents ProcessId
+        rIP <- getE :: GetEvents PortId
+        sM  <- getE :: GetEvents MachineId
+        sP  <- getE :: GetEvents ProcessId
+        sT  <- getE :: GetEvents ThreadId
+        mS  <- getE :: GetEvents MessageSize
+        return  ReceiveMessage { mesTag = toMsgTag tag,
+                                 receiverProcess = rP,
+                                 receiverInport = rIP,
+                                 senderMachine = sM,
+                                 senderProcess = sP,
+                                 senderThread= sT,
+                                 messageSize = mS
+                               })
+ ),
+
+ (FixedSizeParser EVENT_SEND_RECEIVE_LOCAL_MESSAGE
+    (sz_msgtag + 2*sz_procid + 2*sz_tid)
+    (do tag <- getE :: GetEvents Word8
+        sP  <- getE :: GetEvents ProcessId
+        sT  <- getE :: GetEvents ThreadId
+        rP  <- getE :: GetEvents ProcessId
+        rIP <- getE :: GetEvents PortId
+        return SendReceiveLocalMessage { mesTag = toMsgTag tag,
+                                         senderProcess = sP,
+                                         senderThread = sT,
+                                         receiverProcess = rP,
+                                         receiverInport = rIP
+                                       })
+ )]
+
 mercuryParsers = [
  (FixedSizeParser EVENT_MER_START_PAR_CONJUNCTION
     (sz_par_conj_dyn_id + sz_par_conj_static_id)
@@ -619,7 +774,7 @@ getEventBlock parsers = do
       es <- getEventBlock parsers
       return (e:es)
 
-getEventLog :: ErrorT String Get EventLog
+getEventLog :: ExceptT String Get EventLog
 getEventLog = do
     header <- getHeader
     let imap = M.fromList [ (fromIntegral (num t),t) | t <- eventTypes header]
@@ -633,10 +788,31 @@ getEventLog = do
         -- are standard events, and can be used by other runtime systems that
         -- make use of threadscope.
         -}
+
+        -- GHC-7.8.2 uses a different thread block status encoding,
+        -- and therefore requires a different parser for the stop
+        -- event. Later, in GHC-7.8.3, the old encoding was restored.
+        -- GHC-7.8.2 can be recognised by presence and absence of
+        -- events in the header:
+        --   * User markers were added in GHC-7.8 
+        --   * an empty event HACK_BUG_T9003 was added in GHC-7.8.3
+        -- This fix breaks software which uses ghc-events and combines
+        -- user markers with the older stop status encoding. We don't
+        -- know of any such software, though.
+        is_pre77  = M.notMember EVENT_USER_MARKER imap
+        is_ghc782 = M.member EVENT_USER_MARKER imap &&
+                    M.notMember EVENT_HACK_BUG_T9003 imap
+
+        stopParsers = if is_pre77 then pre77StopParsers
+                      else if is_ghc782 then [ghc782StopParser]
+                           else [post782StopParser]
+
         event_parsers = if is_ghc_6
-                            then standardParsers ++ ghc6Parsers
-                            else standardParsers ++ ghc7Parsers
-                                 ++ mercuryParsers ++ perfParsers
+                            then standardParsers ++ ghc6Parsers ++
+                                parRTSParsers sz_old_tid
+                            else standardParsers ++ ghc7Parsers ++
+                                stopParsers ++ parRTSParsers sz_tid ++
+                                mercuryParsers ++ perfParsers
         parsers = mkEventTypeParsers imap event_parsers
     dat <- runReaderT getData (EventParsers parsers)
     return (EventLog header dat)
@@ -644,7 +820,7 @@ getEventLog = do
 readEventLogFromFile :: FilePath -> IO (Either String EventLog)
 readEventLogFromFile f = do
     s <- L.readFile f
-    return $ runGet (do v <- runErrorT $ getEventLog
+    return $ runGet (do v <- runExceptT getEventLog
                         m <- isEmpty
                         m `seq` return v)  s
 
@@ -830,6 +1006,38 @@ showEventInfo spec =
           printf "Unknown event type %d" n
         InternString str sId ->
           printf "Interned string: \"%s\" with id %d" str sId
+        -- events for the parallel RTS
+        Version version ->
+          printf "compiler version is %s" version
+        ProgramInvocation  commandline ->
+          printf "program invocation: %s" commandline
+        EdenStartReceive ->
+          printf "starting to receive"
+        EdenEndReceive ->
+          printf "stop receiving"
+        CreateProcess  process ->
+          printf "creating process %d" process
+        KillProcess process ->
+          printf "killing process %d" process
+        AssignThreadToProcess thread process ->
+          printf "assigning thread %d to process %d" thread process
+        CreateMachine machine realtime ->
+          printf "creating machine %d at %d" machine realtime
+        KillMachine machine ->
+          printf "killing machine %d" machine
+        SendMessage mesTag senderProcess senderThread
+          receiverMachine receiverProcess receiverInport ->
+            printf "sending message with tag %s from process %d, thread %d to machine %d, process %d on inport %d"
+            (show mesTag) senderProcess senderThread receiverMachine receiverProcess receiverInport
+        ReceiveMessage mesTag receiverProcess receiverInport
+          senderMachine senderProcess senderThread messageSize ->
+            printf "receiving message with tag %s at process %d, inport %d from machine %d, process %d, thread %d with size %d"
+            (show mesTag) receiverProcess receiverInport
+            senderMachine senderProcess senderThread messageSize
+        SendReceiveLocalMessage mesTag senderProcess senderThread
+          receiverProcess receiverInport ->
+            printf "sending/receiving message with tag %s from process %d, thread %d to process %d on inport %d"
+            (show mesTag) senderProcess senderThread receiverProcess receiverInport
         MerStartParConjunction dyn_id static_id ->
           printf "Start a parallel conjunction 0x%x, static_id: %d" dyn_id static_id
         MerEndParConjunction dyn_id ->
@@ -875,6 +1083,7 @@ showThreadStopStatus ThreadBlocked  = "thread blocked"
 showThreadStopStatus ThreadFinished = "thread finished"
 showThreadStopStatus ForeignCall    = "making a foreign call"
 showThreadStopStatus BlockedOnMVar  = "blocked on an MVar"
+showThreadStopStatus BlockedOnMVarRead = "blocked reading an MVar"
 showThreadStopStatus BlockedOnBlackHole = "blocked on a black hole"
 showThreadStopStatus BlockedOnRead = "blocked on I/O read"
 showThreadStopStatus BlockedOnWrite = "blocked on I/O write"
@@ -1025,6 +1234,18 @@ eventTypeNum e = case e of
     WallClockTime{} -> EVENT_WALL_CLOCK_TIME
     UnknownEvent {} -> error "eventTypeNum UnknownEvent"
     InternString {} -> EVENT_INTERN_STRING
+    Version {} -> EVENT_VERSION
+    ProgramInvocation {} -> EVENT_PROGRAM_INVOCATION
+    EdenStartReceive {} -> EVENT_EDEN_START_RECEIVE
+    EdenEndReceive {} -> EVENT_EDEN_END_RECEIVE
+    CreateProcess {} -> EVENT_CREATE_PROCESS
+    KillProcess {} -> EVENT_KILL_PROCESS
+    AssignThreadToProcess {} -> EVENT_ASSIGN_THREAD_TO_PROCESS
+    CreateMachine {} -> EVENT_CREATE_MACHINE
+    KillMachine {} -> EVENT_KILL_MACHINE
+    SendMessage {} -> EVENT_SEND_MESSAGE
+    ReceiveMessage {} -> EVENT_RECEIVE_MESSAGE
+    SendReceiveLocalMessage {} -> EVENT_SEND_RECEIVE_LOCAL_MESSAGE
     MerStartParConjunction {} -> EVENT_MER_START_PAR_CONJUNCTION
     MerEndParConjunction _ -> EVENT_MER_STOP_PAR_CONJUNCTION
     MerEndParConjunct _ -> EVENT_MER_STOP_PAR_CONJUNCT
@@ -1072,6 +1293,8 @@ putEventSpec (RunThread t) = do
 
 -- here we assume that ThreadStopStatus fromEnum matches the definitions in
 -- EventLogFormat.h
+-- The standard encoding is used here, which is wrong for eventlogs
+-- produced by GHC-7.8.2 ([Stop status in GHC-7.8.2] in EventTypes.hs
 putEventSpec (StopThread t s) = do
     putE t
     putE $ case s of
@@ -1083,6 +1306,7 @@ putEventSpec (StopThread t s) = do
             ThreadFinished -> 5
             ForeignCall -> 6
             BlockedOnMVar -> 7
+            BlockedOnMVarRead -> 20 -- since GHC-7.8.3
             BlockedOnBlackHole -> 8
             BlockedOnBlackHoleOwnedBy _ -> 8
             BlockedOnRead -> 9
@@ -1300,6 +1524,62 @@ putEventSpec (InternString str id) = do
     putE id
   where len = (fromIntegral (length str) :: Word16) + sz_string_id
 
+putEventSpec (Version s) = do
+    putE (fromIntegral (length s) :: Word16)
+    mapM_ putE s
+
+putEventSpec (ProgramInvocation s) = do
+    putE (fromIntegral (length s) :: Word16)
+    mapM_ putE s
+
+putEventSpec ( EdenStartReceive ) = return ()
+
+putEventSpec ( EdenEndReceive ) = return ()
+
+putEventSpec ( CreateProcess  process ) = do
+    putE process
+
+putEventSpec ( KillProcess process ) = do
+    putE process
+
+putEventSpec ( AssignThreadToProcess thread process ) = do
+    putE thread
+    putE process
+
+putEventSpec ( CreateMachine machine realtime ) = do
+    putE machine
+    putE realtime
+
+putEventSpec ( KillMachine machine ) = do
+    putE machine
+
+putEventSpec ( SendMessage mesTag senderProcess senderThread
+                 receiverMachine receiverProcess receiverInport ) = do
+    putE (fromMsgTag mesTag)
+    putE senderProcess
+    putE senderThread
+    putE receiverMachine
+    putE receiverProcess
+    putE receiverInport
+
+putEventSpec ( ReceiveMessage mesTag receiverProcess receiverInport
+                 senderMachine senderProcess senderThread messageSize ) = do
+    putE (fromMsgTag mesTag)
+    putE receiverProcess
+    putE receiverInport
+    putE senderMachine
+    putE senderProcess
+    putE senderThread
+    putE messageSize
+
+putEventSpec ( SendReceiveLocalMessage mesTag senderProcess senderThread
+                 receiverProcess receiverInport ) = do
+    putE (fromMsgTag mesTag)
+    putE senderProcess
+    putE senderThread
+    putE receiverProcess
+    putE receiverInport
+
 putEventSpec (MerStartParConjunction dyn_id static_id) = do
     putE dyn_id
     putE static_id
